/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */


package com.funambol.mailclient.ui.controller;

import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.ui.view.FunCanvasContactList;
import com.funambol.mailclient.ui.view.FunCanvasInboxMessageList;
import com.funambol.util.Log;
import com.funambol.util.BasicSyncListener;
import com.funambol.util.SyncListener;
import com.funambol.syncml.protocol.SyncML;

/* Note:
 * In multisession mode (when we synchronize more repositories in the same
 * session), we assume that the MailSyncListener is the main listener.
 * The main listener is in charge of performing operations related to the
 * beginning/end of the entire session (such as enabling/disabling sync button).
 */

public class ContactSyncListener extends BasicSyncListener {

    private boolean multiSession;
   
    /** Creates a new instance of MessageListUpdater */
    public ContactSyncListener() {
        multiSession = false;
    }

    public void startSession() {

        Log.debug(this, "Start session");

        if (!multiSession) {
            //DISABLE GetMail Command during sync email
            UIController.enableSyncCommands(false);
        }
        // Update msg
        String msg = Localization.getMessages().SYNC_CONTACTS;
        UIController.updateInboxMessageList(msg);
    }

    public void endSession(int status) {

        Log.debug(this, "End session");

        if (!multiSession) {
            // Reset the title
            UIController.resetInboxTitle();
            //ENABLE GetMail Command at the end of sync email
            UIController.enableSyncCommands(true);

            FunCanvasContactList contactList = UIController.getContactList();
            contactList.initializeContactList();
            contactList.setTitle(contactList.getDefaultTitle());
        }
    }
    
    public void endConnecting(int alertCode) {
        if (alertCode==SyncML.ALERT_CODE_REFRESH_FROM_SERVER) {
            UIController.getContactManager().resetContactList();
        }
    }

    /**
     * This method does not belong to the SyncListener interface.
     * A multi session is a concept introduced by the SyncClient which is able
     * to perform the sync of multiple repositories.
     *
     * @param repositories repositories to be synchronized
     */
    public void startMultiSession(boolean repositories[]) {
        multiSession = true;
    }

    /**
     * This method does not belong to the SyncListener interface.
     * A multi session is a concept introduced by the SyncClient which is able
     * to perform the sync of multiple repositories.
     *
     */
    public void endMultiSession() {

        multiSession = false;
    }
    
    public void endSyncing() {
        Log.debug("[CSL] ending sync!! UIC isFirstSync value=" + UIController.isFirstSync);
        if (UIController.isFirstSync) {
            UIController.isFirstSync=false;
            Log.debug("[CSL] ending sync!! Change Value to false ");
        }
    }
}
