/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.cm.Contact;
import java.util.Vector;

/**
 * an interface to filter contactitems.
 * it consists a match method that returns true if the filter matches the 
 * given item and of a filter method that removes non matching items from the
 * given vector of items
 */
public interface ContactItemFilter {
    
    /**
     * @return true if the filter matches the given contactitem
     *
     * @param c the ContactItem to be filtered
     */
     
    public boolean match(ContactItem c);
    
    /**
     * return a vector with  matching elements.
     * @param fullContactItems the vector with the full list of contact items
     * @param partialContactItems the vector with a partial contact items list, 
     * used if the filter is incremental
     */
    public Vector filter(Vector fullContactItems, Vector partialContactItems)
    throws NoMatchingContactsException;
    
    /**
     * clear the current search.
     * after this method has been called the method match should return true for
     * every ContactItem
     */
    public void clear() ;
    
}
