/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */

package com.funambol.mailclient.ui.view;

import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.ui.controller.Theme;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.loc.Localization;
import com.funambol.util.Log;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.List;

import com.funambol.mail.Folder;

/**
 * a class that display the folders
 */
public class FolderList extends List implements CommandListener{
    
    private MessageManager mm;
    private Command selectCommand;
    
    /** Creates a new instance of FolderList
     * @param folders the array of folderinfo
     */
    public FolderList() {
        super(Localization.getMessages().FOLDER_LIST_TITLE, List.IMPLICIT);
        mm = UIController.messageManager;
        
        append(Localization.getMessages().INBOX_DEFAULT_TITLE, Theme.getINBOX_IMG());
        append(Localization.getMessages().OUTBOX_MESSAGE_LIST_TITLE, Theme.getOUTBOX_IMG());
        append(Localization.getMessages().DRAFT_MESSAGE_LIST_TITLE, Theme.getDRAFT_IMG());
        append(Localization.getMessages().SENT_MESSAGES_LIST_TITLE, Theme.getSENT_IMG());
        
        
        selectCommand=new Command(Localization.getMessages().OPEN_COMMAND, UIController.COMMAND_TYPE, 0);
        addCommand(selectCommand);
        addCommand(UIController.backCommand);
        setSelectCommand(selectCommand);
        setCommandListener(this);
    }
    
    /** setting updated message counters for each folder
     */
    public void updateCounters() {
        set(0, Localization.getMessages().INBOX_DEFAULT_TITLE + 
                getFolderCounter(mm.INBOX), Theme.getINBOX_IMG());
        set(1, Localization.getMessages().OUTBOX_MESSAGE_LIST_TITLE + 
                getFolderCounter(mm.OUTBOX), Theme.getOUTBOX_IMG());
        set(2, Localization.getMessages().DRAFT_MESSAGE_LIST_TITLE +
                getFolderCounter(mm.DRAFTS), Theme.getDRAFT_IMG());
        set(3, Localization.getMessages().SENT_MESSAGES_LIST_TITLE +
                getFolderCounter(mm.SENT), Theme.getSENT_IMG());
    }
    /**
     * commandlistener
     */
    
    //TODO: should we use a UIController method (to decouple this from the
    //MessageManager ? )
    public void commandAction(Command command, Displayable displayable) {
        if (command==selectCommand){
            switch (getSelectedIndex()) {
                case MessageManager.INBOX:
                    UIController.showInboxScreen();
                    break;
                case MessageManager.OUTBOX:
                    UIController.showOutboxScreen(this);
                    break;
                case MessageManager.DRAFTS:
                    UIController.showDraftScreen(this);
                    break;
                case MessageManager.SENT:
                    UIController.showSentScreen(this);
            }
        } else if (command == UIController.backCommand) {
            UIController.showBackScreen();
        }
    }
    
    public void setSelectedFolder(int index) {
        this.setSelectedIndex(index, true);
    }

    private String getFolderCounter(int folder) {
        int counter = mm.getFolder(folder).getMessageCount();
        if (counter > 0) {
            setFont(folder, Font.getFont(Font.FACE_SYSTEM, Font.STYLE_BOLD, Font.SIZE_MEDIUM));
            return " ("+counter+")";
        } else {
            setFont(folder, Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_MEDIUM));
            return "";
        }
    }
}
