/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */

package com.funambol.mailclient.ui.view;

import com.funambol.mail.Folder;
import com.funambol.mail.MailException;
import com.funambol.mail.Message;

import com.funambol.mail.MessageFlags;
import com.funambol.mailclient.Funambol;
import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.sm.SyncClient;
import com.funambol.mailclient.ui.controller.Theme;

import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.loc.Localization;
import com.funambol.util.Log;
import java.util.Vector;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Graphics;
import javax.microedition.rms.RecordStoreException;

public class FunCanvasInboxMessageList extends FunCanvasMessageList implements Runnable {
    private Command openCommand;
    private Command composeCommand;
    private Command flagCommand;
    private Command unflagCommand;
    private Command foldersCommand;
    private Command settingsCommand;
    private Command helpCommand;
    private Command exitCommand;
    private Command syncCommand;
    private Command deleteCommand;
    private FunAd ad;
    private boolean activatedFlagCommand;
    
    private boolean isLoadingMessages=false;
    
    /** Creates a new instance of FunInboxMessageList */
    
    
    public FunCanvasInboxMessageList(Vector msgInfo) {
        super(msgInfo);
        try {
            Folder folder = MessageManager.getInstance().getFolder(MessageManager.INBOX);
            setMinimumDisplayedMessageCount(folder.getMessageCount());
            
        } catch (MailException ex) {
            Log.error(this, "unable to open inbox for message count");
            ex.printStackTrace();
        }
        if(UIController.isSyncPermitted()) {
            enableSyncCommand(true);
        }
        ad = new FunAd();
        
    }
    
    public String getDefaultTitle() {
        return  Localization.getMessages().INBOX_DEFAULT_TITLE;
    }
    
    protected void addCommands() {
        addCommand(composeCommand);
        addCommand(foldersCommand);
        addCommand(settingsCommand);
        addCommand(helpCommand);
        
        //#ifndef JVMExitCommand
        //nokia 6630 adds an exit command by default, so we don't need to add our
        addCommand(exitCommand);
        //#endif
    }
    
    // remove the commands e.g. when we need to go in ad mode
    protected void removeCommands() {
        removeCommand(composeCommand);
        removeCommand(foldersCommand);
        removeCommand(settingsCommand);
        removeCommand(helpCommand);
        
        //#ifndef JVMExitCommand
        //nokia 6630 adds an exit command by default, so we don't need to add our
        removeCommand(exitCommand);
        //#endif
        
    }
    
    /**
     * add the ads command
     */
    
    
    /**
     * called by implementation when a list of elements with lenght >0 is set
     */
    protected void addItemCommands() {
        addCommand(openCommand);
        addCommand(deleteCommand);
        setFireCommand(openCommand);
    }
    
    /**
     * called by implementation when a list of elements with lenght = 0 is set
     */
    protected void removeItemCommands() {
        removeCommand(flagCommand);
        removeCommand(unflagCommand);
        removeCommand(openCommand);
        removeCommand(deleteCommand);
        setFireCommand(null);
    }
    
    
    
    public void commandAction(Command command, Displayable displayable) {
        super.commandAction(command, displayable);
        if (command == composeCommand) {
            UIController.composeNewMessage(UIController.display.getCurrent());
        } else if (command == foldersCommand ) {
            UIController.showFolderList(this);
        }  else if (command == settingsCommand) {
            UIController.showSettings(this);
        } else if (command == helpCommand) {
            UIController.showHelp(this);
            
        } else if (command == exitCommand) {
            
            UIController.midlet.destroyApp(false);
            
        } else if (command == syncCommand) {
            UIController.sync(UIController.isFirstSync);
        }
        // ---- item commands --- //
        else if (command == openCommand) {
            // Use method getFunTitle instead of getTitle
            // for double titlebar in Nokia S40
            String origTitle = getFunTitle();
            setTitle(Localization.getMessages().OPENING_MESSAGE);
            this.repaint();
            this.serviceRepaints();
            UIController.showViewMessageScreen(this, getSelectedMessage());
            setTitle(origTitle);
        } else if (command == flagCommand || command == unflagCommand) {
            if (getSelectedMessage() != null) {
                UIController.changeFollowUpFlag(getSelectedMessage());
                activate(getActiveElementId());
            }
        } else if (command==deleteCommand) {
            deleteMessage();
        }
    }
    
    private void deleteMessage() {
        Message msg = getSelectedMessage();
        if (msg!=null) {
            FunModalPopup popup =
                    new FunModalPopup(Localization.getMessages().POPUP_DELETE_MESSAGE_TITLE,
                    Localization.getMessages().POPUP_DELETE_MESSAGE_TEXT,
                    new DeletePopupAction(msg));
            UIController.showModalPopup(popup, this);
        }
    }
    
    protected void initCommands() {
        int syncCmdType = Command.BACK;
        //#ifdef UseOKCmd
        //#         syncCmdType = UIController.COMMAND_TYPE;
        //#endif
        openCommand =
                new Command(Localization.getMessages().OPEN_COMMAND_LABEL,
                UIController.COMMAND_TYPE,8);
        composeCommand =
                new Command(Localization.getMessages().COMPOSE_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 10);
        foldersCommand =
                new Command(Localization.getMessages().FOLDERS_COMMAND_LABEL,
                UIController.COMMAND_TYPE,40);
        settingsCommand =
                new Command(Localization.getMessages().OPTIONS_COMMAND_LABEL,
                UIController.COMMAND_TYPE,50);
        helpCommand =
                new Command(Localization.getMessages().HELP_COMMAND_LABEL,
                UIController.COMMAND_TYPE,60);
        exitCommand =
                new Command(Localization.getMessages().EXIT_COMMAND_LABEL,
                UIController.COMMAND_TYPE,70);
        syncCommand =
                new Command(Localization.getMessages().SYNC_COMMAND_LABEL, syncCmdType, 5);
        deleteCommand =
                new Command(Localization.getMessages().DELETE_COMMAND_LABEL,
                UIController.COMMAND_TYPE,20);
        flagCommand =
                new Command(Localization.getMessages().FLAG_COMMAND_LABEL,
                UIController.COMMAND_TYPE,30);;
                unflagCommand =
                        new Command(Localization.getMessages().UNFLAG_COMMAND_LABEL,
                        UIController.COMMAND_TYPE,30);;
    }
    
    public void run() {
        load();
    }

    public void load(){
        /* displayInboxMessageList(false); */
        
        
        // loading opened and closed image, they'll be probably needed later and
        // this minimize the empty inbox for devices like nokia 6630
        Theme.getImageFromFlags(MessageFlags.OPENED );
        Theme.getImageFromFlags(0 );
        
        
        isLoadingMessages = true;
        int i = 0;
        MessageManager mm = MessageManager.getInstance();
        Folder f = mm.getFolder(MessageManager.INBOX);
        
        
        
        int tot = f.getMessageCount();
        String[] s = {"Loading messages" , " "};
        
        
        try {
            
            Message msg = mm.getFirstMessage(f);
            while (msg != null) {
                addMessage(msg) ;
                        /*if (i == 10) {
                            UIController.display.setCurrent(this);
                        }*/
                i++;
                s[1] = "(" + i + "/" +tot + ")";
                UIController.midlet.setSplashPhase(s,0);
                msg = mm.getNextMessage(f);
            }
            
            UIController.midlet.setSplashPhase("Opening Inbox",0);
            
        } catch (MailException e) {
            // can happen if a message is deleted while we're in the  cycle
            Log.error("Exception loading message " + i + " from inbox. " +
                    "Some messages have probably been deleted.");
            e.printStackTrace();
        } finally {
            setMinimumDisplayedMessageCount(COUNT_NONE);
            UIController.display.setCurrent(this);
            isLoadingMessages = false;
        }
        
    }
    
    public void setMessageList(Vector msgList) {
        super.setMessageList(msgList);
    }
    
    
    /**
     * display the inbox message list reloading messages from RMS
     * @param resetTitle if true the title is set to the default title
     */
    public void displayInboxMessageList(boolean resetTitle) {
        setMessageList(UIController.getSortedMessages(MessageManager.INBOX));
        
        if (resetTitle) {
            resetTitle();
        }
        UIController.display.setCurrent(this);
    }
    
    protected void handleKey(int keyCode) {
        // handle the shortcut to flag a message with KEY_STAR key (*)
        if (keyCode == KEY_STAR) {
           /* if(activatedFlagCommand)
                commandAction(flagCommand, this);
            else
                commandAction(unflagCommand, this);*/
            if (ad.isEnabled()) {
                UIController.showAdScreen(ad);
            } else {
                if(activatedFlagCommand)
                    commandAction(flagCommand, this);
                else
                    commandAction(unflagCommand, this);
            }
        }
        // handle the shortcut to delete a message with Zero key (0)
        else if (keyCode == KEY_NUM0) {
            deleteMessage();
        }
        
    }
    
    protected void keyPressed(int keyCode) {
        super.keyPressed(keyCode);
        if(!UIController.isSyncInProgress()){
            setTitle(getDefaultTitle());
        }
    }
    
    
    
    protected void activate(int activeElement){
        super.activate(activeElement);
        boolean activeMessageFlag = false;
        
        Message m = getSelectedMessage();
        if ( m != null )  {
            activeMessageFlag = m.getFlags().isSet(MessageFlags.FLAGGED);
            removeCommand(flagCommand);
            removeCommand(unflagCommand);
            if( activeMessageFlag){
                activatedFlagCommand = false;
                addCommand(unflagCommand);
            } else{
                activatedFlagCommand = true;
                addCommand(flagCommand);
            }
        }
    }
    
    
    public void updateMessageFlags(String msgId, MessageFlags flags) {
        int p = search(msgId);
        if (p>=0) {
            // Log.debug(this, "updating message flags for message " + msgId);
            getMessageAt(p).setFlags(flags);
            repaint();
        } else {
            // Log.debug(this, "message " + msgId + " not found");
        }
    }
    /**
     * enable or disable the sync command and the reset inbox command
     * @param enable if true enable the command, if false disable the commands
     */
    public void enableSyncCommand(boolean enable) {
        Log.debug(this, "Enable sync commands: " + enable);
        if (enable) {
            addCommand(syncCommand);
        } else {
            removeCommand(syncCommand);
        }
    }
    /**
     * paint the small ad
     */
    protected int paintAd(Graphics g) {
        int r = ad.paintSmallAd(g);
        return r;
    }
    /**
     * @return true if the inbox is loading messages (i.e. the inbox is in the 
     * run() method)
     */ 
    public boolean isLoadingMessages() {
        return isLoadingMessages;
    }
    
    
    
}
