/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */

package com.funambol.mailclient.ui.view;

import com.funambol.mail.Address;
import com.funambol.mail.MailException;
import com.funambol.mail.Message;
import com.funambol.mailclient.cm.AlreadyExistentContactException;
import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.cm.ContactListFullException;
import com.funambol.mailclient.cm.ContactManagerException;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.util.Log;
import java.util.Vector;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Item;
import javax.microedition.lcdui.ItemCommandListener;
import javax.microedition.lcdui.TextField;

/**
 * shows a list of the recipients of a given message.
 * each recipient can be saved as a contact
 **/
public class FunCanvasRecipientList extends FunCanvasContactList {
    
    private Command saveContactCommand;
    
    /**
     * create a new instance of funRecipientlist with the given message
     */
    public FunCanvasRecipientList(Message message) {
        super(message);
    }
    
    
    
    public void addCommands() {
        addCommand(saveContactCommand);
        addCommand(UIController.cancelCommand);
    }
    
    /**
     * nothing to do here
     */
    protected void removeItemCommands() {
        ;
    }
    
    /**
     * nothing to be done here
     */
    protected void addItemCommands() {
        ;
    }
    
    void initCommands() {
        saveContactCommand =
                new Command(Localization.getMessages().SAVE_COMMAND, UIController.COMMAND_TYPE, 3);
    }
    
    
    /**
     * add the commands the searchbox, and the contacts
     */
    public void initializeContactList() {
    
        try {
            
            Vector list = UiUtils.getMessageAddresses(message);
            Vector contacts = new Vector(list.size());
            Contact c ;
            for (int i = 0; i<list.size(); i++) {
                c=new Contact((Address)list.elementAt(i));
                contacts.addElement(new ContactItem(
                        c,((Address)list.elementAt(i)).getType(), Contact.DEFAULT_EMAIL));
            }
            setContactItemList(contacts);
        } catch (ContactManagerException ex) {
            Log.error("error creating contact from one of the recipients");
            ex.printStackTrace();
        }
    }
    
    public void commandAction(Command command, Displayable d) {
        if (command == saveContactCommand) {
            // we need to reset the contactlist message to clean the
            // recipients
            UIController.getContactList().setMessage(new Message());
            UIController.showAddContactScreen(
                    UIController.display.getCurrent(),getActiveContact());
        }else  if (command==UIController.cancelCommand) {
            UIController.showBackScreen();
        }
    }
    
    protected void enableDisableOKCommand() {
        // nothing to do here!
        ;
    }

    
    void handleKey(int keyCode) {
      // doin' nothing
    }
    
    
    // empty method, returns 0 and does nothing
    protected int drawSubtitle(final Graphics g) {
    // no need to paint stuff here
    return 0;
    }
    
   

    public String getDefaultTitle() {
        
        return Localization.getMessages().RECIPIENT_LIST_TITLE;
    }
    
}
