/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package com.funambol.mailclient.ui.view;

import com.funambol.mail.Address;

/**
 * compare two contacts based only on the state of the contacts
 */
public class StateContactItemComparator implements ContactItemComparator{
    
    /** Creates a new instance of BasicContactItemComparator */
    public StateContactItemComparator() {
        
    }

    /**
     * compare two contactitems using state (to, cc, bcc) 
     * @return a result < 0 if c1 preceds c2, a result > 0 if c2 preceds c1, 
     * return 0 if ci and c2 are equivalent
     */
    public int compare(ContactItem c1, ContactItem c2) {
                
        // if they've the same state we return 0 
        if (c1.getState() == c2.getState() )  {
            return 0;
        }
        
        // if we are here we've different state
        
        // address.to is the first
        if (c1.getState() == Address.TO) {
            return -1;
        }
        
        
        // c1 CC and c2 BCC or NONE
        if ( (c1.getState() == Address.CC) &&  (
                (c2.getState() == Address.BCC) || 
                (c2.getState() == ContactItem.ADDRESS_STATE_NONE )
        ) ) {
            return -1;
        }
        
        // c1 BCC and c2 NONE
        if ( (c1.getState() == Address.BCC) &&  
                (c2.getState() == ContactItem.ADDRESS_STATE_NONE )) {
            return -1;
        }
        
        
        
        // if we arrive here, c2 wins
        return 1; 
        
        
    }
    
}
