/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package com.funambol.mailclient.test;

import com.funambol.mail.Address;
import com.funambol.mail.Message;
import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.cm.ContactManager;
import com.funambol.mailclient.cm.ContactManagerException;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.view.ContactItem;
import com.funambol.mailclient.ui.view.ContactItemCollection;
import com.funambol.util.Log;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;
import javax.microedition.lcdui.Display;
import jmunit.framework.cldc10.TestCase;

public class ContactItemCollectionTest extends TestCase {
    
    ContactManager cm;
    
    /** Creates a new instance of ContactListTest */
    public ContactItemCollectionTest()  {
        super(6, "ContactItemCollection Test");
        // we need this to avoid a nullpointer exception accessing display
        UIController.display = Display.getDisplay(this);
        
        cm = UIController.getContactManager();
        
    }
    
    
    
    
    public void test(int testNumber) throws Throwable {
        
        switch (testNumber) {
            
            case 0:
                testGetContactsFromRMS();
                break;
            case 1:
                testGetContactsFromMessage();
                break;
            case 2:
                break;
            case 3:
                break;
            case 4:
                break;
            case 5:
                break;
            default:
                break;
        }
    }
    
    private void testGetContactsFromRMS() throws Exception {
        Log.info("============================= Test Getting Contacts from rms");
        cm.resetContactList();
        Contact c = createUniqueContact();
        cm.addContact(c);
        
        ContactItemCollection cic = new ContactItemCollection(new Message());
        
        Vector v = cic.getItems();
        assertEquals(v.size(), 1);
        assertEquals( ((ContactItem)v.elementAt(0)).getContact().toString(), c.toString()  );
        Log.info("==================================================== [ OK ]");
        
    }
    
    
    
    
    private void testGetContactsFromMessage() throws Exception {
        Log.info("======================== Test Getting Contacts from message");
        cm.resetContactList();
        Contact c1 = createUniqueContact();
        Contact c2 = createUniqueContact();
        
        
        
        Message m = new Message();
        m.addRecipient(new Address(Address.TO, "address1@mail")) ;
        m.addRecipient(new Address(Address.CC, "address2@mail")) ;
        
        ContactItemCollection cic = new ContactItemCollection(m);
        
        Vector r = cic.getItems();
        
        Log.info("I have  a total of " + r.size() +" contacts!");
        Enumeration e = r.elements();
        assertTrue(r.size() == 2);
        
        
        Address a  = ((ContactItem) e.nextElement()).getAddress();
        assertEquals(a.getVisibleName(), c1.getVisibleName());
        a  = ((ContactItem) e.nextElement()).getAddress();
        assertEquals(a.getVisibleName(), a.getVisibleName());
        
        
        
        
        //assertTrue( (ContactItem)r.elementAt(0))
        
        Log.info("==================================================== [ OK ]");
        
    }
    
    private Contact createUniqueContact() throws  ContactManagerException {
        long now = new Date().getTime();
        return new Contact(
                "Nick-" + now,
                "first-"+ now,
                "last-" + now,
                "email1@" + now,
                "email2@" + now,
                "email3@" + now,
                "00" + now,
                "01" + now,
                "02" + now);
    }
}
