/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package com.funambol.mailclient.test;

import com.funambol.mail.Address;
import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.cm.ContactManagerException;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.view.ContactItem;
import com.funambol.util.Log;
import java.util.Date;
import javax.microedition.lcdui.Display;
import jmunit.framework.cldc10.TestCase;

public class ContactItemTest extends TestCase{
    
    /** Creates a new instance of ContactItemTest */
    public ContactItemTest() {
        super(10, "ContactItem Test");
        UIController.display = Display.getDisplay(this);
        
    }
    
    public void test(int testNumber) throws Throwable {
        Log.info("Starting test number " + testNumber);
        switch (testNumber) {
            
            case 0:
                testConstructors();
                break;
            case 1:
                testToNextAndPreviousEmail();
                break;
            case 2:
                break;
            case 3:
                break;
            case 4:
                break;
            case 5:
                break;
            case 6:
                break;
            case 7:
                break;
            case 8:
                break;
            case 9:
                break;
            default:
                break;
                
                
        }
    }
    
    private void testConstructors() throws Exception {
        
        Log.info("=========================================  Test Constructors");
        Contact c = createUniqueContact();
        
        ContactItem ci = new ContactItem(c);
        Log.info("\t testing default constructor");
        
        assertTrue(ci.getContact().toString().equals(c.toString()));
        assertTrue(ci.getState() == ContactItem.ADDRESS_STATE_NONE);
        assertTrue(ci.getAddress() == null);
        
        int[] state = {Address.TO, Address.CC, Address.BCC};
        int[] emailIndex = {Contact.DEFAULT_EMAIL, Contact.SECONDARY_EMAIL,
        Contact.TERTIARY_EMAIL};
        for (int i=0; i<3; i++) {
            for(int j=0; j<3; j++) {
                Log.info("\t testing constructor with state " + i + " and email " + j);
                ci = new ContactItem(c,state[i],emailIndex[j]);
                assertTrue(ci.getAddress().getType() == state[i]);
                assertTrue(ci.getAddress().getEmail().equals(c.getAddress(
                        state[i], emailIndex[j]).getEmail() ) );
            }
        }
        Log.info("==================================================== [ OK ]");
    }
    
    private Contact createUniqueContact() throws  ContactManagerException {
        long now = new Date().getTime();
        return new Contact(
                "Nick-" + now,
                "first-"+ now,
                "last-" + now,
                "email1@" + now,
                "email2@" + now,
                "email3@" + now,
                "00" + now,
                "01" + now,
                "02" + now);
    }
    
    private void testToNextAndPreviousEmail() throws Exception {
        
        Log.info("========================  Test ToNextEmail / ToPreviousEmail");
        Contact c = createUniqueContact();
        ContactItem ci = new ContactItem(c);
        Log.info("\t testing toNextEmail");
        ci.setState(Address.TO);
        
        assertEquals(c.getDefaultEmail(), ci.getAddress().getEmail());
        ci.toNextEmail();
        assertEquals(c.getEmail_2(), ci.getAddress().getEmail());
        ci.toNextEmail();
        assertEquals(c.getEmail_3(), ci.getAddress().getEmail());
        ci.toNextEmail();
        assertEquals(c.getDefaultEmail(), ci.getAddress().getEmail());
        
        //going back
        ci.toPreviousEmail();
        assertEquals(c.getEmail_3(), ci.getAddress().getEmail());
        ci.toPreviousEmail();
        assertEquals(c.getEmail_2(), ci.getAddress().getEmail());
        ci.toPreviousEmail();
        assertEquals(c.getDefaultEmail(), ci.getAddress().getEmail());
        
        Log.info("==================================================== [ OK ]");
 
    }
    
    
}
