/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/*
 * LoadMessages.java
*/

package com.funambol.mailclient.test;

import com.funambol.mail.MailException;
import com.funambol.mail.Store;
import com.funambol.mailclient.config.ConfigException;
import com.funambol.mailclient.mm.FolderInfo;
import com.funambol.mailclient.mm.MessageInfo;
import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.util.Log;
import java.io.IOException;
import java.util.Date;
import java.util.Vector;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;


import com.funambol.storage.Serializable;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import com.funambol.storage.ObjectStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordStore;

/**
 *
 * A test midlet to measure the loading time of the messages in the inbox.
 *
 * This midlet is supposed to be launched after a sync has been performed with
 * the standard application and some messages are stored in the rms.
 *
 * Pressing the start button will launch the test: the midlet will load the 
 * messages as the application does, and will return the time required 
 * for loading the messages (in msec) .
 *
 */

public class LoadMessages extends MIDlet implements CommandListener {
    
    //items
    private Command startCommand;
    private Command exitCommand;
    private TextField statusTF;
    
    
    
    public void startApp() {
        Display.getDisplay(this).setCurrent(getTheForm());
    }
    
    public void pauseApp() {
    }
    
    public void destroyApp(boolean unconditional) {
        notifyDestroyed();
    }
    
    /**
     * creates the form
     * @return the main form
     */
    private Form getTheForm() {
        
        Form f = new Form(" rms test ");
        startCommand = new Command("Start", Command.OK, 0);
        exitCommand = new Command("Exit", Command.EXIT, 0);
        f.setCommandListener(this);
        f.addCommand(startCommand);
        f.addCommand(exitCommand);
        f.append(getStatusTextField());
        return f;
    }
    
    /**
     * handle the command actions. if startCommand is selected, the test
     * is performed, if exitcommand is selected, the app is closed
     * 
     */
    public void commandAction(Command command, Displayable displayable) {
        
        if (command == startCommand) {
            startTest();
        }
        if (command == exitCommand) {
         destroyApp(false)   ;
        }
        
    }
    
    private TextField getStatusTextField() {
        if (statusTF == null) {
            statusTF=new TextField("Status" ,"",10000, TextField.UNEDITABLE);
        }
        return statusTF;
    }

    /**
     * run the test
     */
    private void startTest() {
        setStatus("Start!");
        try {

            setStatus("getting mm");
            MessageManager mm = MessageManager.getInstance();

            setStatus("init store");
            mm.initStore(false);
          
            setStatus("stating to load folderinfo");
            FolderInfo f= mm.getFolderInfo(Store.INBOX);
            
            long memBefore = Runtime.getRuntime().freeMemory();
            long before = System.currentTimeMillis();

            setStatus("folder loaded at " + before + ", getting list");
            Vector list = f.getMsgList();
            long after = System.currentTimeMillis();

            long memAfter = Runtime.getRuntime().freeMemory();
            int avgSize = (int)(memBefore-memAfter)/list.size();
            Log.error("Average message size: " + avgSize);

            setStatus("loaded "  + list.size() +  " messages in "
                    + (after - before) + " msec" + "\nAverage message size "
                    + avgSize);

        } catch (MailException ex) {
            ex.printStackTrace();
            setStatus("Mailexception");
        }
        
     }
    
    private void setStatus(String s) {
        
        //error level to be sure to log it.
        Log.error("[LoadMessagesTest] " + s);
        getStatusTextField().setString(s);
        
    }
    
}
