/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package com.funambol.mailclient.cm;

/**
 * rapresent a contact manager event
 */

public class ContactManagerEvent {
    
    /**
     * the event code
     */
    private int eventCode;
    
    /**
     * the contact that the event refers to
     */
    private Contact contact;
    
    
    /**
     * code for noop
     */
    public static final int EVENT_CODE_NONE = 0;
    
    /**
     * contact has been added
     */
    public static final int EVENT_CODE_ADD_CONTACT = 1;
    
    /**
     * contact has been updated
     */
    public static final int EVENT_CODE_UPDATE_CONTACT = 2;
    
    /**
     * contact has been deleted
     */
    public static final int EVENT_CODE_DELETE_CONTACT = 3;
    
    /**
     * contact list reset
     */
    public static final int EVENT_CODE_RESET_CONTACT_LIST = 4;
    
    //////////////////////////////////////////////////////////////////
    //    NOTE: if you need to add events constants, be sure to     //
    //    update the comments and the isValidEventCode method!      //
    // ///////////////////////////////////////////////////////////////
    
    
    /**
     *  Creates a new instance of ContactManageEvent with
     *  given  eventcode and contact
     *
     * @param eventCode the event code as one of
     * <ul>
     *  <li>EVENT_CODE_NONE</li>
     *  <li>EVENT_CODE_ADD_CONTACT</li>
     *  <li>EVENT_CODE_UPDATE_CONTACT</li>
     *  <li>EVENT_CODE_DELETE_CONTACT</li>
     *  <li>EVENT_CODE_RESET_CONTACT_LIST</li>
     * </ul>
     */
    public ContactManagerEvent(int eventCode, Contact contact) {
        this.eventCode = eventCode;
        this.contact = contact;
    }
    
    /**
     *  Creates a new instance of ContactManageEvent with
     *  evencCode EVENT_CODE_NONE and null contact.
     */
    public ContactManagerEvent() {
        this(EVENT_CODE_NONE, null);
    }
    /**
     * @return the current contact, or null if no contact has been set
     */
    public Contact getContact() {
        return contact;
    }
    /**
     * set the current contact. null value is valid
     */
    public void setContact(Contact contact) {
        this.contact = contact;
    }
    /**
     *@return the current event code as one of
     * <ul>
     *  <li>EVENT_CODE_NONE</li>
     *  <li>EVENT_CODE_ADD_CONTACT</li>
     *  <li>EVENT_CODE_UPDATE_CONTACT</li>
     *  <li>EVENT_CODE_DELETE_CONTACT</li>
     *  <li>EVENT_CODE_RESET_CONTACT_LIST</li>
     * </ul>
     */
    public int getEventCode() {
        return eventCode;
    }
    
    /**
     *  set the current event code.
     *  @param eventCode the event code to set. must be one of
     *  <ul>
     *  <li>EVENT_CODE_NONE</li>
     *  <li>EVENT_CODE_ADD_CONTACT</li>
     *  <li>EVENT_CODE_UPDATE_CONTACT</li>
     *  <li>EVENT_CODE_DELETE_CONTACT</li>
     *  <li>EVENT_CODE_RESET_CONTACT_LIST</li>
     * </ul>
     * if a different event code is used, this method does nothing
     */
    
    public void setEventCode(int eventCode) {
        if (isValidEventCode(eventCode)) {
            this.eventCode = eventCode;
        }
    }
    
    /**
     * @return true if the given eventcode is valid, i.e. is one of
     *       <ul>
     *  <li>EVENT_CODE_NONE</li>
     *  <li>EVENT_CODE_ADD_CONTACT</li>
     *  <li>EVENT_CODE_UPDATE_CONTACT</li>
     *  <li>EVENT_CODE_DELETE_CONTACT</li>
     *  <li>EVENT_CODE_RESET_CONTACT_LIST</li>
     * </ul>
     */
    private boolean isValidEventCode(int eventCode) {
        return(
                eventCode == EVENT_CODE_ADD_CONTACT ||
                eventCode == EVENT_CODE_DELETE_CONTACT ||
                eventCode == EVENT_CODE_NONE ||
                eventCode == EVENT_CODE_RESET_CONTACT_LIST ||
                eventCode == EVENT_CODE_UPDATE_CONTACT
                );
    }
    
}
